/*
 * AnalyzeCsp - A analyzing tool for random binary symmetric constraint satisfaction problems.
 * Part of the RandomCsp program.
 * Copyright (C) 1998-2001 by J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * $Id: main.cc,v 1.10 2004/02/16 16:06:51 jvhemert Exp $
 *
 */


#include "main.h"

#include <iomanip>
#include <cmath>



/*
 * Prints out help and exits.
 *
 * return: nothing.
 *
 */

void PrintHelpAndDie ()
{
	cout
		<< "Csp Analyzing Tool, version "
		<< CspC::MajorVersion
		<< "."
		<< CspC::MinorVersion
		<< "."
		<< CspC::UpdateVersion
		<< ", Copyright (C) 1998-2002 J.I. van Hemert."
		<< endl
		<< "Compiled on " << __DATE__;

	if (CspC::DebuggingCode)
	{
		cout << " with debugging code.";
	}
	else
	{
		cout << " without debugging code.";
	}
	
	cout
		<< endl << "This program comes as-is and with ABSOLUTELY NO WARRANTY."
		<< endl << endl
		<< "Please check the README for more information on how to use this program"
		<< endl
		<< "and the accompanied source code."
		<< endl << endl
		<< "This program can be used to analyze binary CSP instances" 
		<< endl
		<< "generated by the RandomCsp program." 
		<< endl << endl
		<< "checkcsp <instancefile>" 
		<< endl;
	exit (0);
} // PrintHelp



/*
 * Load a solution from a file and return an array with all the values of the
 * variables.
 *
 * iFilename: filename to read from.
 * iNumberOfVariables: number of variables in the csp problem.
 * oSolution: array with enough space to store all the values of the variables (size=iNumberOfVariables).
 *
 * return: nothing, returns oSolution.
 *
 */

void LoadSolutionFromFile
(
	char const * const iFilename,
	NumberOfVariablesT const iNumberOfVariables,
	ValueT oSolution []
)
{
	unsigned int i = 0;
	char dummy;

	ifstream InputFile;

	InputFile.open (iFilename);

	if (!InputFile.is_open ())
	{
		cerr << "Error: could not open solution file: " << iFilename << "." << endl;
		exit (2);
	}

	do
	{
		if (i < iNumberOfVariables)
			InputFile >> oSolution [i];
		else
			InputFile >> dummy;
		i++;
	} while (!InputFile.eof () && (i < iNumberOfVariables + 3));

	if (i > iNumberOfVariables + 1)
	{
		cerr << "Error: solutions has more variables then problem does." << endl;
		exit (1);
	}

	if (i <= iNumberOfVariables)
	{
		cerr << "Error: solutions has less variables then problem does." << endl;
		exit (1);
	}

	InputFile.close ();

} // LoadSolutionFromFile



int main 
(
	int argc,
	char * argv []
)
{
	if (argc > 2 || (argc == 2 && strcmp(argv[1],"-h") == 0))
		PrintHelpAndDie ();

	StaticCspC csp(argv[1]);

	unsigned int number_of_variables = csp.GetNumberOfVariables();
	

	// domain sizes
	unsigned int total_domainsize = 0;
	unsigned int largest_domainsize = 0;
	for (unsigned int i = 0; i < number_of_variables; i++)
	{
		cout << "Domainsize variable " << i << ": " << csp.GetDomainSize(i) << endl;
		if (csp.GetDomainSize(i) > largest_domainsize)
			largest_domainsize = csp.GetDomainSize(i);
		total_domainsize += csp.GetDomainSize(i);
	}

	// constraints
	unsigned int total_positions = 0;
	
	unsigned int domain_counter[largest_domainsize][largest_domainsize];
	for (unsigned int a = 0; a < largest_domainsize; a++)
		for (unsigned int b = 0; b < largest_domainsize; b++)
				domain_counter[a][b] = 0;
	
	unsigned int conflict_counter[number_of_variables][number_of_variables];
	for (unsigned int a = 0; a < number_of_variables; a++)
		for (unsigned int b = 0; b < number_of_variables; b++)
				conflict_counter[a][b] = 0;
	
	unsigned int positions_counter[number_of_variables][number_of_variables];
	for (unsigned int a = 0; a < number_of_variables; a++)
		for (unsigned int b = 0; b < number_of_variables; b++)
				positions_counter[a][b] = 0;

	for (unsigned int i = 0; i < number_of_variables; i++)
	{
		for (unsigned int j = i+1; j < number_of_variables; j++)
		{
			if (csp.IsConstraint(i,j))
			{
				unsigned int conflicts = 0;
				unsigned int positions = 0;
				for (unsigned int a = 0; a < csp.GetDomainSize(i); a++)
				{
					for (unsigned int b = 0; b < csp.GetDomainSize(j); b++)
					{
						if ( csp.IsConflict(i,j,a,b) )
						{
							conflicts++;
							domain_counter[a][b]++;
						}
						positions++;
					}
				}
				total_positions += positions;
				conflict_counter[i][j] = conflicts;
				positions_counter[i][j] = positions;
				//cout << "Conflicts between variables " << i << " & " << j << ": " << conflicts << " out of " << positions << " t = " << (double) conflicts / (double) positions << endl;
			}
		}
	}


	// Tightness
	cout << "Tightness of constraints:        ";
	for (unsigned int a = 0; a < number_of_variables; a++)
		cout << setw(8) << a;
	cout << endl;
	for (unsigned int a = 0; a < number_of_variables; a++)
	{
		cout << "Tightness of constraints with " << a << ": ";
		for (unsigned int b = 0; b < largest_domainsize; b++)
			if (positions_counter[a][b] == 0)
				cout << " " << setw(7) << "X";
			else
			{
				cout << " " << setiosflags(ios::fixed) <<  setprecision(5) << setw(7) << ((double) conflict_counter[a][b] / (double) positions_counter[a][b]);
			}
		cout << endl;
	}

	// Conflicts
	cout << "Conflicts between elements:  ";
	for (unsigned int a = 0; a < largest_domainsize; a++)
		cout << setw(5) << a;
	cout << endl;
	for (unsigned int a = 0; a < largest_domainsize; a++)
	{
		cout << "Conflicts between element " << a << ": ";
		for (unsigned int b = 0; b < largest_domainsize; b++)
			cout << " " << setw(4) << domain_counter[a][b];
		cout << endl;
	}

	cout << "Number of variables (n): " << number_of_variables << endl;

	// Domainsize
	cout << "Total domainsize: " << total_domainsize << endl;
	double m = (double) total_domainsize / (double) number_of_variables;
	cout << "Domainsize (m) averaged: " << m << endl;

	// Constraints
	cout << "Number of constraints: " << csp.GetNumberOfConstraints() << endl;
	cout << "Maximum number of constraints: " << (unsigned int) ((double) number_of_variables * ((double) number_of_variables -1.0) * 0.5)  << endl;
	double p1 = (double) csp.GetNumberOfConstraints() / ((double) number_of_variables * ((double) number_of_variables -1.00) * 0.5);
	cout << "Ratio of constraints (p1): " << p1 << endl;


	// Total conflicts
	cout << "Number of conflicts: " << csp.GetNumberOfConflictPairs() << endl;
	cout << "Maximum number of conflicts: " << total_positions << endl;
	double p2 = (double) csp.GetNumberOfConflictPairs() / (double) total_positions;
	cout << "Ratio of conflicts (p2): " << p2 << endl;

	cout << "Constrainedness (kappa): " << ((double) number_of_variables - 1.0) * 0.5 * p1 * ( log (1.0 / (1.0 - p2)) / log(m) ) << endl;

	// Number of components (separate graphs)
	unsigned int Sets[csp.GetNumberOfVariables()];
	for (VariableT v = 0; v < csp.GetNumberOfVariables(); v++)
	{
		Sets[v] = v;
	}
	for (VariableT v = 0; v < csp.GetNumberOfVariables(); v++)
	{
		for (VariableT w = v+1; w < csp.GetNumberOfVariables(); w++)
		{
			if (  csp.IsConstraint(v,w) )
			{
				//cout << "X " << v << " " << w << endl;
				Sets[w] = Sets[v];
			}
		}
	}
	unsigned int Components = csp.GetNumberOfVariables();
	for (VariableT v = 0; v < csp.GetNumberOfVariables(); v++)
	{
		//cout << Sets[v] << " ";
		if (Sets[v] != v)
				Components--;
	}
	//cout << endl;
	cout << "Number of components: " << Components << endl;
	if (csp.GetNumberOfVariables() > 1)
	{
		cout << "Ratio of components: " << (double) (Components-1)/(csp.GetNumberOfVariables()-1) << endl;
	}



	return 0;

} // main



// eof main.cc
