/*
 * CspC - A manipulatable datastructure to hold a constraint satisfaction problem.
 * Part of the RandomCsp library.
 * Copyright (C) 1998-2000 by J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * $Id: csp.h,v 1.10 2004/02/16 16:06:52 jvhemert Exp $
 *
 */


#ifndef _CspC_H
#define _CspC_H

using namespace std;

#include "error.h"
#include <iostream>


/**
 * These types can and should be used as a type for variables in your own code.
 *
 * @name Type definitions for CspC
 *
 * @memo General type definitons
 *
 */

//@{
// Conflict between the values of two variables.
typedef bool ConflictT;

// Size of the domain of a variable.
typedef unsigned int DomainSizeT;

// Number of variables in the CSP.
typedef unsigned int NumberOfVariablesT;

// Value of a variable from the domain of the variable.
typedef unsigned int VariableT;

// Value of a variable from the domain of the variable.
typedef unsigned int ValueT;
//@}



/**
 * A base class for implementing Binary ConstrainS Satisfaction Problems.
 *
 * @memo Constraint Satisfaction Problem
 *
 *
 */

class CspC
{
	public:

		/// The storage for a constraint between two variables.
		struct ConstraintS
		{
			ConflictT ** Conflicts;
			ConstraintS * Previous;
			ConstraintS * Next;
		};

		/** The type for a constraint between two variables. */
		typedef ConstraintS * ConstraintT;
		/** */

		static const int MajorVersion = 1;
		static const int MinorVersion = 8;
		static const int UpdateVersion = 0;
#ifdef CSPC_DEBUG
		static const int DebuggingCode = 1;
#else
		static const int DebuggingCode = 0;
#endif

		void ResetConflictChecksCounter()
		{
			conflict_checks = 0;
		}

		unsigned int GetNumberOfConflictChecks()
		{
			return conflict_checks;
		}

		unsigned long GetNumberOfConflictPairs()
		{
			return NumberOfConflictPairs;
		}


		/**
		 * Constructor, allocates memory for all the objects except for the
		 * conflicts.  The conflicts will be allocated as needed when
		 * constraints are added to the
		 * CSP.
		 *
		 * @memo Constructor
		 *
		 * @param iNumberOfVariables number of variables used in the CSP.
		 *
		 * @return nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */

		CspC
		(
			NumberOfVariablesT const iNumberOfVariables
		);



		/**
		 * Copy constructor for CspC. Takes an existing CSP as input and copies
		 * it to the new CSP, including all constraints and conflicts.
		 *
		 * @memo Copy constructor
		 *
		 * @param iCsp existing object of CspC class.
		 *
		 * @return nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */

		CspC
		(
			const CspC & iCsp
		);

		/**
		 * Destroys the left CSP and overwrites it with the right CSP. The left
		 * CSP will receive the constraints and conflicts of the right CSP.
		 *
		 * @memo Operator =
		 *
		 * @param iCsp existing CSP.
		 *
		 * @return new CSP, copied from iCsp.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */

		CspC const & operator=
		(
			const CspC & iCsp
		);



		/**
		 * Destructor deallocates all memory used by the object.
		 *
		 * @memo Destructor
		 *
		 * @return nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */

		virtual ~CspC ();
	


		/**
		 * Get the number of variables present in the CSP.
		 *
		 * @memo Getting the number of variables
		 *
		 * @return number of variables in CSP.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */

		NumberOfVariablesT const GetNumberOfVariables () const;

		/**
		 * Get the size of the domain of a variable.
		 *
		 * @memo Getting the domainsize
		 *
		 * @param iVariable A number corresponding to a variable.
		 *
		 * @return Size of the domain of the given variable.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */

		DomainSizeT const GetDomainSize
		(
			VariableT const iVariable
		) const;

		/**
		 * Set the size of the domain of a variable.
		 *
		 * @memo Setting the domainsize
		 *
		 * @param iVariable A number corresponding to a variabe.
		 * @param iDomainSize size of the domain of iVariable.
		 *
		 * @return nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */

		void const SetDomainSize
		(
			VariableT const iVariable,
			DomainSizeT const iDomainSize
		);



		/**
		 * A way of checking if solutions for the CSP are valid. The input
		 * array with the size of the number of variables in the problem is
		 * checked as a solution for the CSP. If there are no conflicts true is
		 * returned.
		 *
		 * @memo Checking for valid solutions
		 *
		 * @param iSolution an array with values, where the first array index
		 * equals the first variable.
		 *
		 * @return true if the offered solution does not offend a conflict,
		 * false otherwise.
		 *
		 * @author J.I. van Hemert 
		 * @version 1.0
		 *
		 */

		bool const IsValid
		(
			ValueT const iSolution []
		) const;



		/**
		 * A way of finding out how many conflicts a solution violates within
		 * the CSP.  Given a array of size equal to the number of variables,
		 * check how many conflicts this solutions violates in the CSP.
		 *
		 * @memo Getting the number of conflicts
		 *
		 * @param iSolution an array with values, where the first array index
		 * equals the first variable.
		 *
		 * @return number of violated conflicts.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */

		int const GetNumberOfConflicts
		(
			ValueT const iSolution []
		) const;



		/**
		 * Get the number of constraints currently in the CSP.
		 *
		 * @memo Getting the number of constraints
		 *
		 * @return number of constraints in CSP.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */

		unsigned long const GetNumberOfConstraints
		(
		) const;



		/**
		 * Add a constraint between two variables.
		 *
		 * @memo Adding constraints
		 *
		 * @param iVariable1 one of the two variables.
		 * @param iVariable2 another variable.
		 *
		 * @return nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */

		void const AddConstraint
		(
			VariableT const iVariable1,
			VariableT const iVariable2
		);



		/** 
		 * Removes a constraint between two variables. Frees up memory from
		 * conflicts
		 * of the constraint.
		 *
		 * @memo Removing constraints
		 *
		 * @param iVariable1 one of the two variables.
		 * @param iVariable2 another variable.
		 *
		 * @return nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */

		void const RemoveConstraint
		(
			VariableT const iVariable1,
			VariableT const iVariable2
		);



		/**
		 * Check for the existence of a constraint between two variables.
		 *
		 * @memo Testing constraints
		 *
		 * @param iVariable1 one of the two variables.
		 * @param iVariable2 another variable.
		 *
		 * @return true when the constraint exists, else it returns false.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */

		bool const IsConstraint
		(
			 VariableT const iVariable1,
			 VariableT const iVariable2
		) const;



		/**
		 * Add a conflict between the values of two variables.
		 *
		 * @memo Adding conflicts
		 *
		 * @param iVariable1 one of the two variables.
		 * @param iVariable2 another variable.
		 * @param iValue1 value of first variable.
		 * @param iValue2 value of the second variable.
		 *
		 * @return nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */

		void const AddConflict
		(
			VariableT const iVariable1,
			VariableT const iVariable2,
			ValueT const iValue1,
			ValueT const iValue2
		);

		/**
		 * Remove a conflict between the values of two variables.
		 *
		 * @memo Removing conflicts
		 *
		 * @param iVariable1 one of the two variables.
		 * @param iVariable2 another variable.
		 * @param iValue1 value of first variable.
		 * @param iValue2 value of the second variable.
		 *
		 * @return nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */

		void const RemoveConflict
		(
			VariableT const iVariable1,
			VariableT const iVariable2,
			ValueT const iValue1,
			ValueT const iValue2
		);

		/**
		 * Check for an existing conflict between the values of two variables.
		 *
		 * @memo Testing conflicts
		 *
		 * @param iVariable1 one of the two variables.
		 * @param iVariable2 another variable.
		 * @param iValue1 value of first variable.
		 * @param iValue2 value of the second variable.
		 *
		 * @return true for existing constraint with conflict between values,
		 * false for non-existing conflict or non-existing constraint.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */

		bool const IsConflict
		(
			VariableT const iVariable1,
			VariableT const iVariable2,
			ValueT const iValue1,
			ValueT const iValue2
		) const;



		/**
		 * Save the CSP to a file. The fileformat is simple, first the number
		 * of variables is saved, then on the next line all the domainsizes of
		 * the variables are saved with spaces between them. A blank line
		 * follows, after which the conflicts are saved as follows. For every
		 * _possible_ constraint we generate a matrix. First for 1,2 then for 1,3
		 * etc. until n-1,n. The matrix represents the constraint (all zero's if no
		 * constraint exists) with k rows for domainsize k of i in i,j and l columns
		 * for domainsize l of j in i,j.
		 *
		 * @memo Saving a CSP in matrix format
		 *
		 * @param iFilename the filename of the file to save to.
		 *
		 * @return true for success, false for a failure.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */
		
		bool const SaveToFileInMatrix
		(
			char const * const iFilename
		) const;


		/**
		 * Save the CSP to a file. On the second line a keyword denotes whether
		 * this file contains the conflict pairs or the non-conflict pairs.
		 * Then number of variables is saved, on the next line all the
		 * domainsizes of the variables are saved with spaces between them. A
		 * blank line follows, after which the conflicts are saved as follows.
		 * For every non-empty constraint we generate a line that contains first two number denoting the two variables of the constraint followed by a (non-empty) list of domain value pairs. Note that these pairs can either denote conflicts or non-conflicts depending on the keyword mentioned above. Reason for this is to account for efficient saving (if a CSP has a conflict ratio of 99% it is more efficient in disk space to save the non-conflicts).
		 *
		 * @memo Saving a CSP in list format
		 *
		 * @param iFilename the filename of the file to save to.
		 *
		 * @return true for success, false for a failure.
		 *
		 * @author J.I. van Hemert @version 1.0
		 *
		 */
		
		bool const SaveToFileInList ( char const * const iFilename) const;



		/**
		 * Load the CSP from a file.
		 *
		 * @memo Loading a CSP
		 *
		 * @param iFilename the filename of the file to load from.
		 *
		 * @return true for success, false for a failure.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */

		bool const LoadFromFile
		(
			char const * iFilename
		);

		bool const LoadFromStdin
		(
		);

		/**
		 * Check if a CSP file has been touched.
		 *
		 * @memo Testing dirty
		 *
		 * @return false if it is still empty, true if it has been touched.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */

		bool const IsDirty () const;



		/**
		 * Removes all constraints, conflicts and domainsizes from the CSP,
		 * freeing memory for the conflicts, but without freeing memory for the
		 * constraints.  The CSP can be reused for a new CSP, because the
		 * number of variables is not cleared. 
		 *
		 * @memo Resetting the CSP
		 *
		 * @return nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */

		void const Reset ();


		/**
		 * This {\em pure virtual function} has to be implemented by the class
		 * that is inherits from this class. It should produce a Constraint
		 * Satisfaction Problem according to some specific algorithm.
		 *
		 * @memo Virtual member-function
		 *
		 * @return nothing.
		 *
		 */

		//virtual void const ProduceProblem () = 0;
		//virtual void const InitRandomGenerator (int const) = 0;

		/**
		 * This {\em pure virtual function} has to be implemented by the class
		 * that is inherits from this class. It should produce a filename that
		 * is unique for the parameters of problem made by the class it belongs
		 * to.
		 *
		 * @memo Virtual member-function
		 *
		 * @return nothing.
		 *
		 */

		virtual void ProduceFileName
		(
			char *
		) = 0;
	

		/**
		 * This function determines whether this CSP instance is solvable or
		 * not. It uses standard backtracking to find out, which is *very*
		 * slow, especially when you the instance lies near or in the phase
		 * transition.
		 *
		 * @return boolean, false if no solutions or true if at least one solution.
		 *
		 * @version: 1.3
		 *
		 */

		bool const CspC::IsSolvable() const;

	protected:

		// Error handling object.
		ErrorC * Error;

		/** 
		 * Gets a constraint between two variables. Still not sure if we want this to
		 * be public or protected.
		 *
		 * @memo Getting constraints
		 *
		 * @see the structure of a constraint in ConstraintS.
		 *
		 * @param iVariable1 one of the two variables.
		 * @param iVariable2 another variable.
		 *
		 * @return a constraint.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */

		ConstraintT const GetConstraint
		(
			VariableT const iVariable1,
			VariableT const iVariable2
		) const;
		
	private:

		/// To measure the amount of conflict checks performed
		mutable unsigned int conflict_checks;

		// Used by Backtrack
		bool const IsConsistent
		(
			unsigned int current,
			ValueT const iSolution []
		) const;

		bool const Backtrack
		(
			unsigned int current,
			ValueT iSolution []
		) const;

		// Number of variable in the csp.
		NumberOfVariablesT NumberOfVariables;

		// Type for a variable.
		struct VariableS
		{
			DomainSizeT DomainSize;
		};

		// A double indexed array for holding constraints.
		ConstraintT ** Constraints;

		// And a doubly linked list for easy lookup of all constraints.
		ConstraintT ConstraintsList;

		// Counts the number of constraints currently in the CSP.
		unsigned long NumberOfConstraints;

		// Counts the number of pairs of conflicts currently in the CSP.
		unsigned long NumberOfConflictPairs;

		// Counts the number of _possible_ pairs of conflicts currently in the CSP.
		unsigned long NumberOfPossibleConflictPairs;

		// A single indexed array for holding the variables.
		VariableS * Variables;

		// True if csp has been initialised, false if reset.
		bool Dirty;





		/**
		 * Check for the existence of a variable.
		 *
		 * @memo Testing variables
		 *
		 * @param iVariable the variable to check.
		 *
		 * @return true for existence, false for non-existence.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */

		bool const IsValidParameter
		(
			VariableT const iVariable
		) const;

		/**
		 * Check for the existence of a variable and if its value lies in its
		 * domain.
		 *
		 * @memo Testing values
		 *
		 * @param iVariable the variable to check. 
		 * @param iValue value of the variable.
		 *
		 * @return true for existence and valid value, false for non-existence
		 * or non-valid value.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */

		bool const IsValidParameter
		(
			VariableT const iVariable,
			ValueT const iValue
		) const;



		/**
		 * Creates memory needed for holding the CSPs problem. initialises the
		 * CSP by allocating enough memory to hold all the constraints. No
		 * memory is allocated for the conflicts yet.
		 *
		 * @memo Creating a CSP problem
		 *
		 * @param iNumberOfVariables number of variables used in the CSP.
		 *
		 * @return nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */

		void const Create
		(
			NumberOfVariablesT const iNumberOfVariables
		);



		/**
		 * Deallocates memory from the CSP problem. Used in constructors and
		 * destructors.
		 *
		 * @memo Destroying the CSP
		 *
		 * @return nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */

		void const Destroy ();



		/**
		 * Copies an existing CSP to this one by calling Create and copying all
		 * the constraints and conflicts.
		 *
		 * @memo Copying a CSP
		 *
		 * @param iCsp existing CSP.
		 *
		 * @return nothing.
		 *
		 * @author J.I. van Hemert @version 1.0
		 *
		 */

		void const Copy
		(
			CspC const & iCsp
		);



		/**
		 * Swap the two parameters.
		 *
		 * @memo Swap function
		 *
		 * @param iVariable1 First parameter.
		 * @param iVariable2 Second parameter.
		 * 
		 * @return nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 1.0
		 *
		 */

		void const Swap
		(
			VariableT & iVariable1,
			VariableT & iVariable2
		) const;

		bool const LoadFromFileInMatrix
		(
			istream & InputFile
		);

		bool const LoadFromFileInList
		(
			istream & InputFile
		);



}; // CspC


#endif

// eof csp.h
