/*
 * RandomCspA - An abstract class for random binary csp generators
 * Part of the RandomCsp library.
 * Copyright (C) 1998-2000 by J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * $Id: randomcsp.h,v 1.3 2004/02/16 16:06:53 jvhemert Exp $
 *
 */


#ifndef _RandomCspA_H
#define _RandomCspA_H


#include <cstdlib>
#include "csp.h"


/**
 * This class is an abstract class that can be extended to a class capable of
 * producing Random Binary Constraint Satisfaction Problems. The class should implement
 * the ProduceProblem function.
 *
 * @memo Random Constraint Satisfaction Problems
 *
 * @author J.I. van Hemert @version 1.1
 *
 */

class RandomCspA: 
	public CspC
{
	public:



		/**
		 * Constructor copies all the parameters for further usage.
		 *
		 * @memo Constructor
		 *
		 * @param iNumberOfVariables Number of variables in CSP.
		 * @param iDomainSize Size of domain of each variable *or* maximum size of domain.
		 * @param iRandomDomainSize If true than a the domainsize will be varied over the variables.
		 * @param iTightness Determines percentage of conflicts that are generated.
		 * @param iConnectivity Percentage of constraints that are generated.
		 *
		 * @return nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 1.2
		 *
		 */

		RandomCspA
		(
			NumberOfVariablesT const iNumberOfVariables
		): CspC(iNumberOfVariables)
		{};



		/**
		 * For C++'s sake we have to implement the virtual destructor.
		 *
		 * @memo Destructor
		 *
		 * @return nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 1.2
		 *
		 */

		~RandomCspA () {};



		/**
		 * Initialise the randomgenerator using a seed.
		 *
		 * @memo Initialising the randomgenerator
		 *
		 * @param iRandomSeed Seed used for initialising the random generator.
		 *
		 * @param return nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 1.2
		 *
		 */

		void const InitRandomGenerator
		(
			int const iRandomSeed
		);



		/**
		 * Produce a constraint satisfaction method using the method of Patrick
		 * Prosser, which he mentiones in his article 'Binary Constraint
		 * Satisfaction Problems: Some are Harder than Others' in '11th
		 * European Conference on Artificial Inteligence (ECAI94)' edited by A.
		 * Cohn and published in 1994 by * John Wiley \& Sons, Ltd.
		 *
		 * @memo Producing a Random CSP
		 *
		 * @return nothing.
		 * 
		 * @author J.I. van Hemert
		 * @version 1.2
		 *
		 */

		virtual void const ProduceProblem () =0;

	protected:

		int GetSeed()
		{
			return Seed;
		}

	private:

		// Seed for the randomgenerator
		static int Seed;

}; // RandomCspA


#endif

// eof randomcsp.h
