/*
 * RandomCsp - A program for generating random constraint satisfaction problems.
 * Part of the RandomCsp package.
 * Copyright (C) 1998-2003 by J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * $Id: main.cc,v 1.9 2003/03/04 17:32:42 jvhemert Exp $
 *
 */



#include "main.h"

typedef enum FileFormatE
{
	Matrix,
	List
} FileFormatT;

typedef enum MethodE
{
	ModelA,
	ModelB,
	ModelC,
	ModelD,
	ModelE,
	ModelF,
	ModelG,
	ModelRB,
	modelcounts
} MethodT;

/*
 * Print general information and all the program arguments to stdout.
 *
 * return: nothing.
 *
 */

void PrintHelp ()
{
	cout
		<< "Random Csp Generator, version "
		<< CspC::MajorVersion
		<< "."
		<< CspC::MinorVersion
		<< "."
		<< CspC::UpdateVersion
		<< ", Copyright (C) 1998-2003 J.I. van Hemert."
		<< endl
		<< "Compiled on " << __DATE__;

	if (CspC::DebuggingCode)
	{
		cout << " with debugging code.";
	}
	else
	{
		cout << " without debugging code.";
	}
	
	cout
		<< endl << "This program comes as-is and with ABSOLUTELY NO WARRANTY."
		<< endl << endl
		<< "Please check the README for more information on how to use this program"
		<< endl
		<< "and the accompanied source code."
		<< endl << endl
		<< "Options:" << endl
		<< "    -cr    Chance of generating a constraint (model a-d) or p of the" << endl
		<< "           geometric distribution (model g) (default: 0.5)" << endl
		<< "    -dn    Size of domain of variables (integer n = 20)" << endl
		<< "    -ms    Method used for generating the random problem {a-g,r} (default: e)" << endl
		<< "    -fs    String used as filename for the output" << endl
		<< "    -h     Print help and version" << endl
		<< "    -Os    Output format m for matrix and l list (default: m)" << endl
		<< "    -pn    Number of problems to generate (default: 1)" << endl
		<< "    -q     Keep quiet" << endl
		<< "    -r     Choose random size of domain for each variable" << endl
		<< "    -sn    Seed for the random generator (default: 1)" << endl
		<< "    -tr    Chance of generating a conflict; tightness (default: 0.5)" << endl
		<< "    -S     Produce only solvable instances, can take forever!" << endl
		<< "    -T     Test every instance to see if it has a solution, can take very long!" << endl
		<< "    -vn    Number of variables (default: 20)" << endl
			;

} // PrintHelp



int main (int argc, char * argv [])
{
	int NumberOfVariables = 20;
	int DomainSize = 20;
	bool RandomDomainSize = false;
	bool SolutionTest = false;
	bool OnlySolvables = false;
	bool KeepQuiet = false;
	double Tightness = 0.5;
	double Connectivity = 0.5;
	int Seed = 1;
	char * FileName = 0;
	char * Temp = 0;
	char * FormatString = 0;
	FileFormatT fileformat = Matrix;
	MethodT method = ModelE;
	unsigned int NumberOfProblems = 1;
	RandomCspA * gen;

	// This one is used by getopt() to find out which argument its parsing.
	int option_index;
	int Arg = 0;

	option_index = 0;
	while (Arg > -1)
	{
		Arg = getopt (argc, argv, "qTShv:d:rt:c:s:f:p:m:O:");
		switch (Arg)
		{
			case 'h':
				PrintHelp ();
				exit (0);
				break;
			case 'v':
				NumberOfVariables = atoi(optarg);
				break;
			case 'd':
				DomainSize = atoi(optarg);
				break;
			case 'r':
				RandomDomainSize = true;
				break;
			case 't':
				Tightness = atof(optarg);
				break;
			case 'c':
				Connectivity = atof(optarg);
				break;
			case 's':
				Seed = atoi(optarg);
				break;
			case 'f':
				FileName = strdup (optarg);
				break;
			case 'p':
				NumberOfProblems = atoi (optarg);
				break;
			case 'O':
				switch (optarg[0])
				{
					case 'm':
						fileformat = Matrix;
						break;
					case 'l':
						fileformat = List;
						break;
					default:
						cerr << optarg << " is not a valid option for -o." << endl;
						exit(2);
				}
				break;
			case 'm':
				switch (optarg[0])
				{
					case 'a':
						method = ModelA;
						break;
					case 'b':
						method = ModelB;
						break;
					case 'c':
						method = ModelC;
						break;
					case 'd':
						method = ModelD;
						break;
					case 'e':
						method = ModelE;
						break;
					case 'f':
						method = ModelF;
						break;
					case 'g':
						method = ModelG;
						break;
					case 'r':
						method = ModelRB;
						break;
					default:
						cerr << optarg << " is not a valid option for -m." << endl;
						exit(1);
				}
				break;
			case 'q':
				KeepQuiet = true;
				break;
			case 'S':
				OnlySolvables = true;
				break;
			case 'T':
				SolutionTest = true;
				break;
			case -1:
				// getopt returns -1 after last argument.
				break;
			default:
				cerr << "Use '" << argv[0] << " -h' for help." << endl;
				exit (1);
		}
	}

	switch (method)
	{
		case ModelA:
			gen = new ModelARandomCspC (NumberOfVariables, DomainSize, RandomDomainSize, Tightness, Connectivity);
			break;
		case ModelB:
			gen = new ModelBRandomCspC (NumberOfVariables, DomainSize, RandomDomainSize, Tightness, Connectivity);
			break;
		case ModelC:
			gen = new ModelCRandomCspC (NumberOfVariables, DomainSize, RandomDomainSize, Tightness, Connectivity);
			break;
		case ModelD:
			gen = new ModelDRandomCspC (NumberOfVariables, DomainSize, RandomDomainSize, Tightness, Connectivity);
			break;
		case ModelE:
			gen = new ModelERandomCspC (NumberOfVariables, DomainSize, RandomDomainSize, Tightness);
			break;
		case ModelF:
			gen = new ModelFRandomCspC (NumberOfVariables, DomainSize, RandomDomainSize, Tightness, Connectivity);
			break;
		case ModelG:
			gen = new ModelGRandomCspC (NumberOfVariables, DomainSize, RandomDomainSize, Tightness, Connectivity);
			break;
		case ModelRB:
			gen = new ModelRBRandomCspC (NumberOfVariables, DomainSize, RandomDomainSize, Tightness, Connectivity);
			break;
		default:
			cerr << "This method for generating does not exist." << endl;
			exit(2);
	}

	gen->InitRandomGenerator (Seed);

	if (FileName == 0)
	{
		FileName = new char [100];
		gen->ProduceFileName (FileName);
	}

	{
		Temp = new char [120];
		FormatString = new char [12];
		sprintf (FormatString, "%%0%ui.", (unsigned int) log10 ((double)NumberOfProblems) + 1);

				 
		for (unsigned int i = 0; i < NumberOfProblems; i++)
		{
			sprintf (Temp, FormatString, i);

			if (!KeepQuiet) fprintf(stderr, "Creating problem number %i", i);
			do
			{
				if (!KeepQuiet) fprintf(stderr, ".");
				gen->ProduceProblem ();
			}
			while (OnlySolvables && ! gen->IsSolvable());
			if (!KeepQuiet) fprintf(stderr, "\n");

			if (SolutionTest && ! OnlySolvables)
			{
				if ( gen->IsSolvable() )
				{
					Temp = strcat(Temp, "solvable.");
				}
				else
				{
					Temp = strcat(Temp, "unsolvable.");
				}
			}

			if (OnlySolvables)
			{
				Temp = strcat(Temp, "solvable.");
			}


			Temp = strcat (Temp, FileName);
			if (fileformat == List)
				gen->SaveToFileInList (Temp);
			if (fileformat == Matrix)
				gen->SaveToFileInMatrix (Temp);
					
		}
	}

	delete FileName;

	delete gen;

	return 0;
} // main


// eof main.cc
